@@ -10,9 +10,11 @@
 
 #include <cplusplus/CppDocument.h>
 #include <utils/environment.h>
-#include <utils/futuresynchronizer.h>
-#include <utils/qrcparser.h>
 #include <utils/filepath.h>
+#include <utils/futuresynchronizer.h>
+#include <utils/id.h>
+#include <utils/qrcparser.h>
+#include <utils/synchronizedvalue.h>
 
 #include <QFuture>
 #include <QHash>
@@ -24,8 +26,6 @@
 
 QT_FORWARD_DECLARE_CLASS(QTimer)
 
-namespace ProjectExplorer { class Project; }
-
 namespace QmlJS {
 
 class Snapshot;
@@ -45,9 +45,11 @@ public:
         AllQrcResources
     };
 
+    using ProjectBase = QObject; // This is a ProjectExplorer::Project in the using code.
+
     struct ProjectInfo
     {
-        QPointer<ProjectExplorer::Project> project;
+        QPointer<ProjectBase> project;
         QList<Utils::FilePath> sourceFiles;
         PathsAndLanguages importPaths;
         QList<Utils::FilePath> activeResourceFiles;
@@ -113,33 +115,37 @@ public:
     static void writeWarning(const QString &msg);
     static WorkingCopy workingCopy();
     static Utils::FilePath qmllsForBinPath(const Utils::FilePath &binPath, const QVersionNumber &v);
+    static Utils::FilePath qmlformatForBinPath(const Utils::FilePath &binPath, const QVersionNumber &v);
 
     QmlJS::Snapshot snapshot() const;
     QmlJS::Snapshot newestSnapshot() const;
     QThreadPool *threadPool();
 
+    QSet<Utils::FilePath> scannedPaths() const;
+    void removeFromScannedPaths(const PathsAndLanguages &pathsAndLanguages);
+
     void activateScan();
     void updateSourceFiles(const QList<Utils::FilePath> &files, bool emitDocumentOnDiskChanged);
     void fileChangedOnDisk(const Utils::FilePath &path);
     void removeFiles(const QList<Utils::FilePath> &files);
     QStringList qrcPathsForFile(const Utils::FilePath &file,
                                 const QLocale *locale = nullptr,
-                                ProjectExplorer::Project *project = nullptr,
+                                ProjectBase *project = nullptr,
                                 QrcResourceSelector resources = AllQrcResources);
     QStringList filesAtQrcPath(const QString &path, const QLocale *locale = nullptr,
-                               ProjectExplorer::Project *project = nullptr,
+                               ProjectBase *project = nullptr,
                                QrcResourceSelector resources = AllQrcResources);
     QMap<QString, QStringList> filesInQrcPath(const QString &path,
                                               const QLocale *locale = nullptr,
-                                              ProjectExplorer::Project *project = nullptr,
+                                              ProjectBase *project = nullptr,
                                               bool addDirs = false,
                                               QrcResourceSelector resources = AllQrcResources);
     Utils::FilePath fileToSource(const Utils::FilePath &file);
 
     QList<ProjectInfo> projectInfos() const;
-    bool containsProject(ProjectExplorer::Project *project) const;
-    ProjectInfo projectInfo(ProjectExplorer::Project *project) const;
-    void updateProjectInfo(const ProjectInfo &pinfo, ProjectExplorer::Project *p);
+    bool containsProject(ProjectBase *project) const;
+    ProjectInfo projectInfo(ProjectBase *project) const;
+    void updateProjectInfo(const ProjectInfo &pinfo, ProjectBase *p);
 
     void updateDocument(const QmlJS::Document::Ptr& doc);
     void updateLibraryInfo(const Utils::FilePath &path, const QmlJS::LibraryInfo &info);
@@ -168,7 +174,7 @@ public:
 
     void setDefaultVContext(const ViewerContext &vContext);
     virtual ProjectInfo defaultProjectInfo() const;
-    virtual ProjectInfo defaultProjectInfoForProject(ProjectExplorer::Project *project,
+    virtual ProjectInfo defaultProjectInfoForProject(ProjectBase *project,
                                                      const Utils::FilePaths &hiddenRccFolders) const;
 
     // Blocks until all parsing threads are done. Use for testing only!
@@ -187,11 +193,12 @@ public:
                                 bool emitDocChanged, bool libOnly = true, bool forceRescan = false);
 
     virtual void resetCodeModel();
-    void removeProjectInfo(ProjectExplorer::Project *project);
+    void removeProjectInfo(ProjectBase *project);
     void maybeQueueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr &doc);
 
     QFuture<void> refreshSourceFiles(const QList<Utils::FilePath> &sourceFiles,
                                      bool emitDocumentOnDiskChanged);
+    void waitForFinished();
 
 signals:
     void documentUpdated(QmlJS::Document::Ptr doc);
@@ -210,7 +217,7 @@ protected:
     virtual void writeMessageInternal(const QString &msg) const;
     virtual WorkingCopy workingCopyInternal() const;
     virtual void addTaskInternal(const QFuture<void> &result, const QString &msg,
-                                 const char *taskId) const;
+                                 const Utils::Id taskId) const;
 
     static void parseLoop(QSet<Utils::FilePath> &scannedPaths,
                           QSet<Utils::FilePath> &newLibraries,
@@ -233,26 +240,35 @@ protected:
     void maybeScan(const PathsAndLanguages &importPaths);
     void updateImportPaths();
     void loadQmlTypeDescriptionsInternal(const QString &path);
-    void setDefaultProject(const ProjectInfo &pInfo, ProjectExplorer::Project *p);
+    void setDefaultProject(const ProjectInfo &pInfo, ProjectBase *p);
+    void cancelAllThreads();
 
 private:
     void joinAllThreads(bool cancelOnWait = false);
-    void iterateQrcFiles(ProjectExplorer::Project *project,
+    void iterateQrcFiles(ProjectBase *project,
                          QrcResourceSelector resources,
                          const std::function<void(Utils::QrcParser::ConstPtr)> &callback);
     ViewerContext getVContext(const ViewerContext &vCtx, const Document::Ptr &doc, bool limitToProject) const;
 
-    mutable QMutex m_mutex;
-    QmlJS::Snapshot m_validSnapshot;
-    QmlJS::Snapshot m_newestSnapshot;
-    PathsAndLanguages m_allImportPaths;
-    QList<Utils::FilePath> m_applicationPaths;
-    QList<Utils::FilePath> m_defaultImportPaths;
-    QmlJS::QmlLanguageBundles m_activeBundles;
-    QmlJS::QmlLanguageBundles m_extendedBundles;
-    QHash<Dialect, QmlJS::ViewerContext> m_defaultVContexts;
-    bool m_shouldScanImports = false;
-    QSet<Utils::FilePath> m_scannedPaths;
+    struct SyncedData
+    {
+        SyncedData(const QList<Utils::FilePath> &defaultImportPaths);
+        QmlJS::Snapshot m_validSnapshot;
+        QmlJS::Snapshot m_newestSnapshot;
+        PathsAndLanguages m_allImportPaths;
+        QList<Utils::FilePath> m_applicationPaths;
+        QList<Utils::FilePath> m_defaultImportPaths;
+        QmlJS::QmlLanguageBundles m_activeBundles;
+        QmlJS::QmlLanguageBundles m_extendedBundles;
+        QHash<Dialect, QmlJS::ViewerContext> m_defaultVContexts;
+        bool m_shouldScanImports = false;
+        QSet<Utils::FilePath> m_scannedPaths;
+        ProjectBase *m_defaultProject = nullptr;
+        ProjectInfo m_defaultProjectInfo;
+        QHash<ProjectBase *, ProjectInfo> m_projects;
+        QMultiHash<Utils::FilePath, ProjectBase *> m_fileToProject;
+    };
+    Utils::SynchronizedValue<SyncedData> m_syncedData;
 
     QTimer *m_updateCppQmlTypesTimer = nullptr;
     QTimer *m_asyncResetTimer = nullptr;
@@ -261,15 +277,12 @@ private:
     Utils::QrcCache m_qrcCache;
     QHash<Utils::FilePath, QString> m_qrcContents;
 
-    CppDataHash m_cppDataHash;
-    QHash<QString, QList<CPlusPlus::Document::Ptr>> m_cppDeclarationFiles;
-    mutable QMutex m_cppDataMutex;
-
-    // project integration
-    QMap<ProjectExplorer::Project *, ProjectInfo> m_projects;
-    ProjectInfo m_defaultProjectInfo;
-    ProjectExplorer::Project *m_defaultProject = nullptr;
-    QMultiHash<Utils::FilePath, ProjectExplorer::Project *> m_fileToProject;
+    struct SyncedCppData
+    {
+        CppDataHash m_cppDataHash;
+        QHash<QString, QList<CPlusPlus::Document::Ptr>> m_cppDeclarationFiles;
+    };
+    Utils::SynchronizedValue<SyncedCppData> m_syncedCppData;
 
     PluginDumper *m_pluginDumper = nullptr;
 
@@ -278,6 +291,37 @@ private:
 
     bool m_indexerDisabled = false;
     QThreadPool m_threadPool;
+
+private:
+    QList<Utils::FilePath> importPathsNames(const SyncedData &lockedData) const;
+
+    static bool findNewQmlApplicationInPath(
+        const Utils::FilePath &path,
+        const Snapshot &snapshot,
+        ModelManagerInterface *modelManager,
+        QSet<Utils::FilePath> *newLibraries,
+        Utils::SynchronizedValue<ModelManagerInterface::SyncedData>::unique_lock &lock);
+
+    static void findNewLibraryImports(const Document::Ptr &doc,
+                                      const Snapshot &snapshot,
+                                      ModelManagerInterface *modelManager,
+                                      Utils::FilePaths *importedFiles,
+                                      QSet<Utils::FilePath> *scannedPaths,
+                                      QSet<Utils::FilePath> *newLibraries,
+                                      Utils::SynchronizedValue<SyncedData>::unique_lock *lock);
+
+    static bool findNewQmlLibraryInPath(const Utils::FilePath &path,
+                                        const Snapshot &snapshot,
+                                        ModelManagerInterface *modelManager,
+                                        QList<Utils::FilePath> *importedFiles,
+                                        QSet<Utils::FilePath> *scannedPaths,
+                                        QSet<Utils::FilePath> *newLibraries,
+                                        bool ignoreMissing,
+                                        Utils::SynchronizedValue<SyncedData>::unique_lock *lock);
+
+    void updateLibraryInfo(const Utils::FilePath &path,
+                           const QmlJS::LibraryInfo &info,
+                           Utils::SynchronizedValue<SyncedData>::unique_lock &lock);
 };
 
 } // namespace QmlJS
