@@ -18,6 +18,12 @@ class QFuture;
 
 namespace Tasking {
 
+class Do;
+class For;
+class Group;
+class GroupItem;
+using GroupItems = QList<GroupItem>;
+
 Q_NAMESPACE_EXPORT(TASKING_EXPORT)
 
 // WorkflowPolicy:
@@ -181,6 +187,17 @@ class Storage final : public StorageBase
 {
 public:
     Storage() : StorageBase(Storage::ctor(), Storage::dtor()) {}
+#if __cplusplus >= 201803L // C++20: Allow pack expansion in lambda init-capture.
+    template <typename ...Args>
+    Storage(const Args &...args)
+        : StorageBase([...args = args] { return new StorageStruct{args...}; }, Storage::dtor()) {}
+#else // C++17
+    template <typename ...Args>
+    Storage(const Args &...args)
+        : StorageBase([argsTuple = std::tuple(args...)] {
+            return std::apply([](const Args &...arguments) { return new StorageStruct{arguments...}; }, argsTuple);
+        }, Storage::dtor()) {}
+#endif
     StorageStruct &operator*() const noexcept { return *activeStorage(); }
     StorageStruct *operator->() const noexcept { return activeStorage(); }
     StorageStruct *activeStorage() const {
@@ -207,13 +224,12 @@ public:
         : m_type(Type::Storage)
         , m_storageList{storage} {}
 
-    GroupItem(const Loop &loop) : GroupItem(GroupData{{}, {}, {}, loop}) {}
-
     // TODO: Add tests.
-    GroupItem(const QList<GroupItem> &children) : m_type(Type::List) { addChildren(children); }
+    GroupItem(const GroupItems &children) : m_type(Type::List) { addChildren(children); }
     GroupItem(std::initializer_list<GroupItem> children) : m_type(Type::List) { addChildren(children); }
 
 protected:
+    GroupItem(const Loop &loop) : GroupItem(GroupData{{}, {}, {}, loop}) {}
     // Internal, provided by CustomTask
     using InterfaceCreateHandler = std::function<TaskInterface *(void)>;
     // Called prior to task start, just after createHandler
@@ -257,11 +273,9 @@ protected:
     GroupItem(const TaskHandler &handler)
         : m_type(Type::TaskHandler)
         , m_taskHandler(handler) {}
-    void addChildren(const QList<GroupItem> &children);
+    void addChildren(const GroupItems &children);
 
     static GroupItem groupHandler(const GroupHandler &handler) { return GroupItem({handler}); }
-    static GroupItem parallelLimit(int limit) { return GroupItem({{}, limit}); }
-    static GroupItem workflowPolicy(WorkflowPolicy policy) { return GroupItem({{}, {}, policy}); }
 
     // Checks if Function may be invoked with Args and if Function's return type is Result.
     template <typename Result, typename Function, typename ...Args,
@@ -275,11 +289,14 @@ protected:
     }
 
 private:
+    TASKING_EXPORT friend Group operator>>(const For &forItem, const Do &doItem);
     friend class ContainerNode;
     friend class TaskNode;
     friend class TaskTreePrivate;
+    friend class ParallelLimitFunctor;
+    friend class WorkflowPolicyFunctor;
     Type m_type = Type::Group;
-    QList<GroupItem> m_children;
+    GroupItems m_children;
     GroupData m_groupData;
     QList<StorageBase> m_storageList;
     TaskHandler m_taskHandler;
@@ -288,34 +305,38 @@ private:
 class TASKING_EXPORT ExecutableItem : public GroupItem
 {
 public:
-    ExecutableItem withTimeout(std::chrono::milliseconds timeout,
-                               const std::function<void()> &handler = {}) const;
-    ExecutableItem withLog(const QString &logName) const;
+    Group withTimeout(std::chrono::milliseconds timeout,
+                      const std::function<void()> &handler = {}) const;
+    Group withLog(const QString &logName) const;
     template <typename SenderSignalPairGetter>
-    ExecutableItem withCancel(SenderSignalPairGetter &&getter) const
-    {
-        const auto connectWrapper = [getter](QObject *guard, const std::function<void()> &trigger) {
-            const auto senderSignalPair = getter();
-            QObject::connect(senderSignalPair.first, senderSignalPair.second, guard, [trigger] {
-                trigger();
-            }, static_cast<Qt::ConnectionType>(Qt::QueuedConnection | Qt::SingleShotConnection));
-        };
-        return withCancelImpl(connectWrapper);
-    }
+    Group withCancel(SenderSignalPairGetter &&getter, std::initializer_list<GroupItem> postCancelRecipe = {}) const;
+    template <typename SenderSignalPairGetter>
+    Group withAccept(SenderSignalPairGetter &&getter) const;
 
 protected:
     ExecutableItem() = default;
     ExecutableItem(const TaskHandler &handler) : GroupItem(handler) {}
 
 private:
-    ExecutableItem withCancelImpl(
+    TASKING_EXPORT friend Group operator!(const ExecutableItem &item);
+    TASKING_EXPORT friend Group operator&&(const ExecutableItem &first,
+                                                    const ExecutableItem &second);
+    TASKING_EXPORT friend Group operator||(const ExecutableItem &first,
+                                                    const ExecutableItem &second);
+    TASKING_EXPORT friend Group operator&&(const ExecutableItem &item, DoneResult result);
+    TASKING_EXPORT friend Group operator||(const ExecutableItem &item, DoneResult result);
+
+    Group withCancelImpl(
+        const std::function<void(QObject *, const std::function<void()> &)> &connectWrapper,
+        const GroupItems &postCancelRecipe) const;
+    Group withAcceptImpl(
         const std::function<void(QObject *, const std::function<void()> &)> &connectWrapper) const;
 };
 
 class TASKING_EXPORT Group : public ExecutableItem
 {
 public:
-    Group(const QList<GroupItem> &children) { addChildren(children); }
+    Group(const GroupItems &children) { addChildren(children); }
     Group(std::initializer_list<GroupItem> children) { addChildren(children); }
 
     // GroupData related:
@@ -327,8 +348,6 @@ public:
     static GroupItem onGroupDone(Handler &&handler, CallDoneIf callDoneIf = CallDoneIf::SuccessOrError) {
         return groupHandler({{}, wrapGroupDone(std::forward<Handler>(handler)), callDoneIf});
     }
-    using GroupItem::parallelLimit;  // Default: 1 (sequential). 0 means unlimited (parallel).
-    using GroupItem::workflowPolicy; // Default: WorkflowPolicy::StopOnError.
 
 private:
     template <typename Handler>
@@ -350,15 +369,17 @@ private:
     template <typename Handler>
     static GroupDoneHandler wrapGroupDone(Handler &&handler)
     {
-        static constexpr bool isDoneResultType = std::is_same_v<Handler, DoneResult>;
-        // R, V, D stands for: Done[R]esult, [V]oid, [D]oneWith
+        static constexpr bool isDoneResultType = std::is_same_v<std::decay_t<Handler>, DoneResult>;
+        // R, B, V, D stands for: Done[R]esult, [B]ool, [V]oid, [D]oneWith
         static constexpr bool isRD = isInvocable<DoneResult, Handler, DoneWith>();
         static constexpr bool isR = isInvocable<DoneResult, Handler>();
+        static constexpr bool isBD = isInvocable<bool, Handler, DoneWith>();
+        static constexpr bool isB = isInvocable<bool, Handler>();
         static constexpr bool isVD = isInvocable<void, Handler, DoneWith>();
         static constexpr bool isV = isInvocable<void, Handler>();
-        static_assert(isDoneResultType || isRD || isR || isVD || isV,
+        static_assert(isDoneResultType || isRD || isR || isBD || isB || isVD || isV,
             "Group done handler needs to take (DoneWith) or (void) as an argument and has to "
-            "return void or DoneResult. Alternatively, it may be of DoneResult type. "
+            "return void, bool or DoneResult. Alternatively, it may be of DoneResult type. "
             "The passed handler doesn't fulfill these requirements.");
         return [handler](DoneWith result) {
             if constexpr (isDoneResultType)
@@ -367,15 +388,44 @@ private:
                 return std::invoke(handler, result);
             if constexpr (isR)
                 return std::invoke(handler);
+            if constexpr (isBD)
+                return toDoneResult(std::invoke(handler, result));
+            if constexpr (isB)
+                return toDoneResult(std::invoke(handler));
             if constexpr (isVD)
                 std::invoke(handler, result);
             else if constexpr (isV)
                 std::invoke(handler);
-            return result == DoneWith::Success ? DoneResult::Success : DoneResult::Error;
+            return toDoneResult(result == DoneWith::Success);
         };
     }
 };
 
+template <typename SenderSignalPairGetter>
+Group ExecutableItem::withCancel(SenderSignalPairGetter &&getter,
+                                 std::initializer_list<GroupItem> postCancelRecipe) const
+{
+    const auto connectWrapper = [getter](QObject *guard, const std::function<void()> &trigger) {
+        const auto senderSignalPair = getter();
+        QObject::connect(senderSignalPair.first, senderSignalPair.second, guard, [trigger] {
+            trigger();
+        }, static_cast<Qt::ConnectionType>(Qt::QueuedConnection | Qt::SingleShotConnection));
+    };
+    return withCancelImpl(connectWrapper, postCancelRecipe);
+}
+
+template <typename SenderSignalPairGetter>
+Group ExecutableItem::withAccept(SenderSignalPairGetter &&getter) const
+{
+    const auto connectWrapper = [getter](QObject *guard, const std::function<void()> &trigger) {
+        const auto senderSignalPair = getter();
+        QObject::connect(senderSignalPair.first, senderSignalPair.second, guard, [trigger] {
+            trigger();
+        }, static_cast<Qt::ConnectionType>(Qt::QueuedConnection | Qt::SingleShotConnection));
+    };
+    return withAcceptImpl(connectWrapper);
+}
+
 template <typename Handler>
 static GroupItem onGroupSetup(Handler &&handler)
 {
@@ -388,10 +438,11 @@ static GroupItem onGroupDone(Handler &&handler, CallDoneIf callDoneIf = CallDone
     return Group::onGroupDone(std::forward<Handler>(handler), callDoneIf);
 }
 
+// Default: 1 (sequential). 0 means unlimited (parallel).
 TASKING_EXPORT GroupItem parallelLimit(int limit);
-TASKING_EXPORT GroupItem workflowPolicy(WorkflowPolicy policy);
 
-TASKING_EXPORT extern const GroupItem nullItem;
+// Default: WorkflowPolicy::StopOnError.
+TASKING_EXPORT GroupItem workflowPolicy(WorkflowPolicy policy);
 
 TASKING_EXPORT extern const GroupItem sequential;
 TASKING_EXPORT extern const GroupItem parallel;
@@ -405,11 +456,43 @@ TASKING_EXPORT extern const GroupItem stopOnSuccessOrError;
 TASKING_EXPORT extern const GroupItem finishAllAndSuccess;
 TASKING_EXPORT extern const GroupItem finishAllAndError;
 
-class TASKING_EXPORT Forever final : public Group
+TASKING_EXPORT extern const GroupItem nullItem;
+TASKING_EXPORT extern const ExecutableItem successItem;
+TASKING_EXPORT extern const ExecutableItem errorItem;
+
+class TASKING_EXPORT For final
 {
 public:
-    Forever(const QList<GroupItem> &children) : Group({LoopForever(), children}) {}
-    Forever(std::initializer_list<GroupItem> children) : Group({LoopForever(), children}) {}
+    explicit For(const Loop &loop) : m_loop(loop) {}
+
+private:
+    TASKING_EXPORT friend Group operator>>(const For &forItem, const Do &doItem);
+
+    Loop m_loop;
+};
+
+class When;
+
+class TASKING_EXPORT Do final
+{
+public:
+    explicit Do(const GroupItems &children) : m_children(children) {}
+    explicit Do(std::initializer_list<GroupItem> children) : m_children(children) {}
+
+private:
+    TASKING_EXPORT friend Group operator>>(const For &forItem, const Do &doItem);
+    TASKING_EXPORT friend Group operator>>(const When &whenItem, const Do &doItem);
+
+    GroupItem m_children;
+};
+
+class TASKING_EXPORT Forever final : public ExecutableItem
+{
+public:
+    explicit Forever(const GroupItems &children)
+        { addChildren({ For (LoopForever()) >> Do { children } } ); }
+    explicit Forever(std::initializer_list<GroupItem> children)
+        { addChildren({ For (LoopForever()) >> Do { children } } ); }
 };
 
 // Synchronous invocation. Similarly to Group - isn't counted as a task inside taskCount()
@@ -418,26 +501,20 @@ class TASKING_EXPORT Sync final : public ExecutableItem
 public:
     template <typename Handler>
     Sync(Handler &&handler) {
-        addChildren({ onGroupSetup(wrapHandler(std::forward<Handler>(handler))) });
+        addChildren({ onGroupDone(wrapHandler(std::forward<Handler>(handler))) });
     }
 
 private:
     template <typename Handler>
-    static GroupSetupHandler wrapHandler(Handler &&handler) {
-        // R, V stands for: Done[R]esult, [V]oid
+    static auto wrapHandler(Handler &&handler) {
+        // R, B, V stands for: Done[R]esult, [B]ool, [V]oid
         static constexpr bool isR = isInvocable<DoneResult, Handler>();
+        static constexpr bool isB = isInvocable<bool, Handler>();
         static constexpr bool isV = isInvocable<void, Handler>();
-        static_assert(isR || isV,
-            "Sync handler needs to take no arguments and has to return void or DoneResult. "
+        static_assert(isR || isB || isV,
+            "Sync handler needs to take no arguments and has to return void, bool or DoneResult. "
             "The passed handler doesn't fulfill these requirements.");
-        return [handler] {
-            if constexpr (isR) {
-                return std::invoke(handler) == DoneResult::Success ? SetupResult::StopWithSuccess
-                                                                   : SetupResult::StopWithError;
-            }
-            std::invoke(handler);
-            return SetupResult::StopWithSuccess;
-        };
+        return handler;
     }
 };
 
@@ -501,19 +578,25 @@ private:
     static InterfaceDoneHandler wrapDone(Handler &&handler) {
         if constexpr (std::is_same_v<Handler, TaskDoneHandler>)
             return {}; // User passed {} for the done handler.
-        static constexpr bool isDoneResultType = std::is_same_v<Handler, DoneResult>;
-        // R, V, T, D stands for: Done[R]esult, [V]oid, [T]ask, [D]oneWith
+        static constexpr bool isDoneResultType = std::is_same_v<std::decay_t<Handler>, DoneResult>;
+        // R, B, V, T, D stands for: Done[R]esult, [B]ool, [V]oid, [T]ask, [D]oneWith
         static constexpr bool isRTD = isInvocable<DoneResult, Handler, const Task &, DoneWith>();
         static constexpr bool isRT = isInvocable<DoneResult, Handler, const Task &>();
         static constexpr bool isRD = isInvocable<DoneResult, Handler, DoneWith>();
         static constexpr bool isR = isInvocable<DoneResult, Handler>();
+        static constexpr bool isBTD = isInvocable<bool, Handler, const Task &, DoneWith>();
+        static constexpr bool isBT = isInvocable<bool, Handler, const Task &>();
+        static constexpr bool isBD = isInvocable<bool, Handler, DoneWith>();
+        static constexpr bool isB = isInvocable<bool, Handler>();
         static constexpr bool isVTD = isInvocable<void, Handler, const Task &, DoneWith>();
         static constexpr bool isVT = isInvocable<void, Handler, const Task &>();
         static constexpr bool isVD = isInvocable<void, Handler, DoneWith>();
         static constexpr bool isV = isInvocable<void, Handler>();
-        static_assert(isDoneResultType || isRTD || isRT || isRD || isR || isVTD || isVT || isVD || isV,
+        static_assert(isDoneResultType || isRTD || isRT || isRD || isR
+                                       || isBTD || isBT || isBD || isB
+                                       || isVTD || isVT || isVD || isV,
             "Task done handler needs to take (const Task &, DoneWith), (const Task &), "
-            "(DoneWith) or (void) as arguments and has to return void or DoneResult. "
+            "(DoneWith) or (void) as arguments and has to return void, bool or DoneResult. "
             "Alternatively, it may be of DoneResult type. "
             "The passed handler doesn't fulfill these requirements.");
         return [handler](const TaskInterface &taskInterface, DoneWith result) {
@@ -528,6 +611,14 @@ private:
                 return std::invoke(handler, result);
             if constexpr (isR)
                 return std::invoke(handler);
+            if constexpr (isBTD)
+                return toDoneResult(std::invoke(handler, *adapter.task(), result));
+            if constexpr (isBT)
+                return toDoneResult(std::invoke(handler, *adapter.task()));
+            if constexpr (isBD)
+                return toDoneResult(std::invoke(handler, result));
+            if constexpr (isB)
+                return toDoneResult(std::invoke(handler));
             if constexpr (isVTD)
                 std::invoke(handler, *adapter.task(), result);
             else if constexpr (isVT)
@@ -536,11 +627,26 @@ private:
                 std::invoke(handler, result);
             else if constexpr (isV)
                 std::invoke(handler);
-            return result == DoneWith::Success ? DoneResult::Success : DoneResult::Error;
+            return toDoneResult(result == DoneWith::Success);
         };
     }
 };
 
+template <typename Task>
+class SimpleTaskAdapter final : public TaskAdapter<Task>
+{
+public:
+    SimpleTaskAdapter() { this->connect(this->task(), &Task::done, this, &TaskInterface::done); }
+    void start() final { this->task()->start(); }
+};
+
+// A convenient helper, when:
+// 1. Task is derived from QObject.
+// 2. Task::start() method starts the task.
+// 3. Task::done(DoneResult) signal is emitted when the task is finished.
+template <typename Task>
+using SimpleCustomTask = CustomTask<SimpleTaskAdapter<Task>>;
+
 class TASKING_EXPORT TaskTree final : public QObject
 {
     Q_OBJECT
@@ -561,10 +667,8 @@ public:
     // Don't use it in main thread. To be used in non-main threads or in auto tests.
     DoneWith runBlocking();
     DoneWith runBlocking(const QFuture<void> &future);
-    static DoneWith runBlocking(const Group &recipe,
-        std::chrono::milliseconds timeout = std::chrono::milliseconds::max());
-    static DoneWith runBlocking(const Group &recipe, const QFuture<void> &future,
-        std::chrono::milliseconds timeout = std::chrono::milliseconds::max());
+    static DoneWith runBlocking(const Group &recipe);
+    static DoneWith runBlocking(const Group &recipe, const QFuture<void> &future);
 
     int asyncCount() const;
     int taskCount() const;
@@ -632,6 +736,9 @@ private:
 using TaskTreeTask = CustomTask<TaskTreeTaskAdapter>;
 using TimeoutTask = CustomTask<TimeoutTaskAdapter>;
 
+TASKING_EXPORT ExecutableItem timeoutTask(const std::chrono::milliseconds &timeout,
+                                          DoneResult result = DoneResult::Error);
+
 } // namespace Tasking
 
 QT_END_NAMESPACE
