@@ -19,8 +19,6 @@ namespace Utils {
 
 enum class StreamMode { Reader, Writer, Transfer };
 
-enum class StreamResult { FinishedWithSuccess, FinishedWithError };
-
 class QTCREATOR_UTILS_EXPORT FileStreamer final : public QObject
 {
     Q_OBJECT
@@ -38,26 +36,18 @@ public:
     // Only for Writer mode
     void setWriteData(const QByteArray &writeData);
 
-    StreamResult result() const;
+    Tasking::DoneResult result() const;
 
     void start();
     void stop();
 
 signals:
-    void done();
+    void done(Tasking::DoneResult result);
 
 private:
     class FileStreamerPrivate *d = nullptr;
 };
 
-class FileStreamerTaskAdapter : public Tasking::TaskAdapter<FileStreamer>
-{
-public:
-    FileStreamerTaskAdapter() { connect(task(), &FileStreamer::done, this,
-                [this] { emit done(task()->result() == StreamResult::FinishedWithSuccess); }); }
-    void start() override { task()->start(); }
-};
-
-using FileStreamerTask = Tasking::CustomTask<FileStreamerTaskAdapter>;
+using FileStreamerTask = Tasking::SimpleCustomTask<FileStreamer>;
 
 } // namespace Utils
