@@ -3,6 +3,8 @@
 
 #pragma once
 
+#include "environmentaspect.h"
+#include "runconfigurationaspects.h"
 #include "runcontrol.h"
 
 namespace ProjectExplorer {
@@ -12,18 +14,22 @@ class PROJECTEXPLORER_EXPORT CustomExecutableRunConfiguration : public RunConfig
     Q_OBJECT
 
 public:
-    CustomExecutableRunConfiguration(Target *target, Utils::Id id);
-    explicit CustomExecutableRunConfiguration(Target *target);
+    CustomExecutableRunConfiguration(BuildConfiguration *bc, Utils::Id id);
+    explicit CustomExecutableRunConfiguration(BuildConfiguration *bc);
 
     QString defaultDisplayName() const;
 
 private:
-    Runnable runnable() const override;
-    bool isEnabled() const override;
+    bool isEnabled(Utils::Id) const override;
     Tasks checkForIssues() const override;
 
     void configurationDialogFinished();
-    Utils::FilePath executable() const;
+
+    EnvironmentAspect environment{this};
+    ExecutableAspect executable{this};
+    ArgumentsAspect arguments{this};
+    WorkingDirectoryAspect workingDir{this};
+    TerminalAspect terminal{this};
 };
 
 class CustomExecutableRunConfigurationFactory : public FixedRunConfigurationFactory
@@ -32,10 +38,4 @@ public:
     CustomExecutableRunConfigurationFactory();
 };
 
-class CustomExecutableRunWorkerFactory : public RunWorkerFactory
-{
-public:
-    CustomExecutableRunWorkerFactory();
-};
-
 } // namespace ProjectExplorer
